<?php
/**
 * This file is responsible to customize single product page
 *
 * PHP version 7
 *
 * @category WordPress_Theme
 * @package  ThemeCustomization
 *
 * @author  WProDevs <info@wprodevs.com>
 * @license https://www.gnu.org/licenses/gpl-3.0.html GPLv3 or later
 *
 * @link none
 */

namespace WooCommerce;

/**
 * This class is used to customize single product page
 *
 * @category WordPress_Theme
 * @package  ThemeCustomization
 *
 * @author  WProDevs <info@wprodevs.com>
 * @license https://www.gnu.org/licenses/gpl-3.0.html GPLv3 or later
 *
 * @link none
 */
final class SingleProduct
{
    use Traits\HelperFunctions;

    /**
     * Stores a private copy of self instance used for singleton
     *
     * @var stdClass
     */
    private static $_instance;

    /**
     * Flag to represent whether there are related products
     *
     * @var boolean
     */
    private $_hasRelatedProducts = true;

    /**
     * Enqueue CSS for single product page
     */
    public function __construct()
    {
        add_action('wp_enqueue_scripts', [$this, 'enqueueResources']);
    }

    /**
     * Enqueue static resources
     *
     * @return none
     */
    public function enqueueResources()
    {
        if (is_product()) {
            wp_enqueue_style('wpro-wc-single-product', get_stylesheet_directory_uri() . '/css/single-product.css', ['child-style']);
            wp_enqueue_style('wpro-social-share', get_stylesheet_directory_uri() . '/css/social-share.css', ['wpro-wc-single-product']);
            wp_enqueue_style('wpro-tabs', '//cdn.jsdelivr.net/gh/k-ivan/tabs-slider/dist/css/tabs.min.css', ['wpro-wc-single-product']);
            wp_enqueue_script('wpro-tabs', '//cdn.jsdelivr.net/gh/k-ivan/tabs-slider/dist/js/tabsSlider.js', null, false, true);
            wp_enqueue_script('wpro-single-product', get_stylesheet_directory_uri() . '/js/single-product.js', ['wpro-tabs'], false, true);
        }
    }

    /**
     * Customize single product page by overriding WooCommerce hooks
     *
     * @return non
     */
    public function customizeSingleProductPage()
    {
        global $avada_woocommerce;
        /**
         * Remove existing hooks
         */
        remove_action('woocommerce_before_single_product_summary', 'woocommerce_show_product_sale_flash', 10);
        remove_action('woocommerce_after_single_product_summary', 'woocommerce_output_product_data_tabs', 10);
        remove_action('woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20);
        remove_action('woocommerce_after_single_product_summary', [$avada_woocommerce, 'output_related_products'], 15);
        remove_action('woocommerce_single_product_summary', 'woocommerce_template_single_meta', 40);

        /**
         * Add new hooks
         */
        add_action('woocommerce_single_product_summary', 'woocommerce_show_product_sale_flash', 4);
        add_action('woocommerce_single_product_summary', [$this, 'showProductBrand'], 4);
        add_filter('woocommerce_format_sale_price', [$this, 'formatSalePrice'], 10, 3);
        add_action('woocommerce_after_add_to_cart_button', [$this, 'addwishListButton'], 10);
        add_action('woocommerce_before_single_variation', [$this, 'socialShareLinks'], 20);
        add_action('woocommerce_single_product_summary', 'woocommerce_output_product_data_tabs', 70);
        /**
         * Helper hooks for related products by attribute
         */
        add_filter('woocommerce_shortcode_products_query', [$this, 'excludeCurrentProductFromQuery']);
        add_action('woocommerce_shortcode_product_attribute_loop_no_results', [$this, 'noRelatedProductsFound'], 10);
        $tabbed = apply_filters('is_tabbed_view', true);
        if (!$tabbed) {
            /**
             * Non-tabbed
             */
            add_action('woocommerce_after_single_product_summary', [$this, 'showRelatedProductsByAttribute'], 20, 0);
            add_action('woocommerce_after_single_product_summary', [$this, 'showRecentProducts'], 20, 0);
            add_action('woocommerce_after_single_product_summary', [$this, 'showRecentlyViewed'], 20, 0);
        } else {
            /**
             * Tabbed
             */
            add_action('woocommerce_after_single_product_summary', [$this, 'showTabs'], 20);
        }
        /**
         * Remove reviews tab
         */
        add_filter('woocommerce_product_tabs', [$this, 'removeTabs'], 98);
    }

    /**
     * Override plugin hooks
     *
     * @return none
     */
    public function pluginOverrides()
    {
        if (class_exists('\\YITH_WRVP_Frontend')) {
            $yith_wrvp = \YITH_WRVP_Frontend::get_instance();
            remove_action('woocommerce_after_single_product_summary', [$yith_wrvp, 'print_shortcode'], 30);
        }
    }

    /**
     * Show product brand attribute before product title
     *
     * @return none
     */
    public function showProductBrand()
    {
        global $product;
        $brand = $product->get_attribute('pa_brand');
        printf('<header>%1$s</header>', $brand);
    }

    /**
     * [formatSalePrice description]
     *
     * @param string $price         Formatted, strike through regular price
     * @param string $regular_price Formatted regular price
     * @param string $sale_price    Formatted sale price
     *
     * @return string
     */
    public function formatSalePrice($price, $regular_price, $sale_price)
    {
        return "<ins>{$sale_price}</ins><del>{$regular_price}</del>";
    }

    /**
     * Add YITH Wish List plugin
     *
     * @return none
     */
    public function addwishListButton()
    {
        echo do_shortcode('[yith_wcwl_add_to_wishlist]');
    }

    /**
     * Add social media share URLs
     *
     * @return none
     */
    public function socialShareLinks()
    {
        $this->_getTemplate(
            'social-share.php',
            false,
            [
                'atts' => [
                    'facebook' => true,
                    'twitter'  => true,
                    'email'    => true,
                    'whatsapp' => true
                ]
            ]
        );
    }

    /**
     * Show related products section
     *
     * @param boolean $echo Whether to echo the result
     *
     * @return none
     */
    public function showRelatedProductsByAttribute($echo = true)
    {
        global $product;
        $brand  = $product->get_attribute('pa_brand');
        $brands = explode(',', $brand);
        if (is_array($brands)) {
            $brands = array_map('trim', $brands);
        }
        $related_products = do_shortcode("[product_attribute attribute=pa_brand terms='{$brand}']");
        if ($this->_hasRelatedProducts) {
            if (is_array($brands) && !empty($brands)) {
                $permalinks = '';
                foreach ($brands as $brand) {
                    $term = get_term_by('name', $brand, 'pa_brand');
                    $link = get_term_link($term);
                    if (!is_wp_error($link)) {
                        $permalinks .= sprintf('<a href="%1$s" class="view-more">View All %2$s</a>', $link, $brand);
                    }
                }
                $permalink = $permalinks;
            } else {
                $term = get_term_by('name', $brand, 'pa_brand');
                $link = get_term_link($term);
                if (!is_wp_error($link)) {
                    $permalink = sprintf('<a href="%1$s" class="view-more">View All</a>', $link);
                }
            }
            $result = '<section class="related-products">';
            $result .= sprintf('<h2>More %1$s</h2>', $brand);
            $result .= $permalink;
            $result .= $related_products;
            $result .= '</section>';
            if (!$echo) {
                return $result;
            }
            echo $result;
        } elseif (!$echo) {
            return false;
        }
    }

    /**
     * Remove current Product from WP_Query attributes
     *
     * @param array $atts Default attributes
     *
     * @return array
     */
    public function excludeCurrentProductFromQuery($atts)
    {
        if (is_product()) {
            global $product;
            $atts['posts_per_page'] = 8;
            $atts['post__not_in']   = [$product->get_id()];
        }

        return $atts;
    }

    /**
     * Set flag variable if no related products are found
     *
     * @return boolean
     */
    public function noRelatedProductsFound()
    {
        if (is_product()) {
            $this->_hasRelatedProducts = false;
        }
    }

    /**
     * Show recently added products
     *
     * @param boolean $echo Whether to echo the result
     *
     * @return none
     */
    public function showRecentProducts($echo = true)
    {
        $result = '<section class="related-products">';
        $result .= '<h2>New In</h2>';
        $result .= do_shortcode('[recent_products]');
        $result .= '</section>';
        if (!$echo) {
            return $result;
        }
        echo $result;
    }

    /**
     * Show recently viewed products
     *
     * @param boolean $echo Whether to echo the result
     *
     * @return none
     */
    public function showRecentlyViewed($echo = true)
    {
        if (class_exists('\\YITH_WRVP_Frontend')) {
            $yith_wrvp = \YITH_WRVP_Frontend::get_instance();
            $result    = '<section class="related-products">';
            $title     = sprintf('<h2>%s</h2>', get_option('yith-wrvp-section-title'));
            $atts      = [
                'posts_per_page' => 8,
                'title'          => '',
            ];
            if ($echo) {
                $title = '';
                unset($atts['title']);
            }
            $yith_res = trim($yith_wrvp->similar_products($atts));
            $result .= $title;
            $result .= $yith_res;
            $result .= '</section>';
            if (!$echo) {
                return empty($yith_res) ? false : $result;
            }
            echo $result;
        }
    }

    /**
     * Create and show tabbed view
     *
     * @return none
     */
    public function showTabs()
    {
        global $product;
        $brand = $product->get_attribute('pa_brand');
        $tabs  = [
            [
                'title'   => "More {$brand}",
                'content' => [$this, 'showRelatedProductsByAttribute'],
                'param'   => [false]
            ],
            [
                'title'   => 'New In',
                'content' => [$this, 'showRecentProducts'],
                'param'   => [false]
            ],
            [
                'title'   => 'Recently Viewed',
                'content' => [$this, 'showRecentlyViewed'],
                'param'   => [false]
            ],
        ];
        $this->_getTemplate(
            'single-product-tabs.php',
            false,
            [
                'tabs' => $tabs
            ]
        );
    }

    /**
     * Remove product data tabs
     *
     * @param array $tabs Default data tabs
     *
     * @see https://docs.woocommerce.com/document/editing-product-data-tabs/
     *
     * @return array
     */
    public function removeTabs($tabs)
    {
        unset($tabs['reviews']);

        return $tabs;
    }

    /**
     * Singleton implementation to only allow 1 instance
     *
     * @return stdClass
     */
    public static function getInstance()
    {
        if (!isset(self::$_instance) || !self::$_instance instanceof self) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }
}
