<?php
/**
 * This file is responsible to customize WooCommerce
 *
 * PHP version 7
 *
 * @category WordPress_Theme
 * @package  ThemeCustomization
 *
 * @author  WProDevs <info@wprodevs.com>
 * @license https://www.gnu.org/licenses/gpl-3.0.html GPLv3 or later
 *
 * @link none
 */

namespace WooCommerce;

/**
 * This class is used to customize WooCommerce
 *
 * @category WordPress_Theme
 * @package  ThemeCustomization
 *
 * @author  WProDevs <info@wprodevs.com>
 * @license https://www.gnu.org/licenses/gpl-3.0.html GPLv3 or later
 *
 * @link none
 */
final class WooCommerceCustomization
{
    /**
     * Stores a private copy of self instance used for singleton
     *
     * @var stdClass
     */
    private static $_instance;

    /**
     * Stores a private copy of SingleProduct instance used for singleton
     *
     * @var stdClass
     */
    private static $_singleProduct;

    /**
     * Stores a private copy of SingleProduct instance used for singleton
     *
     * @var stdClass
     */
    private static $_woocommerceShortcodes;

    /**
     * Setup initial process after instantiation
     */
    public function __construct()
    {
        $this->_initialize();
    }

    /**
     * Initialize WooCommerce Customizations
     *
     * @return none
     */
    private function _initialize()
    {
        $this->_loadClasses();
        $this->_loadCustomizations();
    }

    /**
     * Load additional customization classes
     *
     * @return none
     */
    private function _loadClasses()
    {
        self::$_singleProduct         = SingleProduct::getInstance();
        self::$_woocommerceShortcodes = WooCommerceShortcodes::getInstance();
    }

    /**
     * Call various class functions for customizations
     *
     * @return none
     */
    private function _loadCustomizations()
    {
        self::$_singleProduct->customizeSingleProductPage();
        add_action('init', [$this, 'delayedCustomizations']);
        add_action('woocommerce_attribute_show_in_nav_menus', [$this, 'showAttributesInNavigation']);
        add_action('woocommerce_product_single_add_to_cart_text', [$this, 'changeAddToCartText']);
    }

    /**
     * Customizations to be run after all plugins have loaded
     *
     * @return none
     */
    public function delayedCustomizations()
    {
        self::$_singleProduct->pluginOverrides();
    }

    /**
     * Allow product attributes to appear in navigation
     *
     * @param boolean $default Default value
     * @param string  $name    Attribute slug
     *
     * @return bool
     */
    public function showAttributesInNavigation($default = false, $name = '')
    {
        return true;
    }

    /**
     * Change Add To Cart button text
     *
     * @return string
     */
    public function changeAddToCartText()
    {
        return 'Add To Basket';
    }

    /**
     * Return blank/null/empty value
     *
     * @return null|string
     */
    function returnBlank()
    {
        return null;
    }

    /**
     * Singleton implementation to only allow 1 instance
     *
     * @return stdClass
     */
    public static function getInstance()
    {
        if (!isset(self::$_instance) || !self::$_instance instanceof self) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }
}
