<?php
/**
 * This file is responsible to add custom WooCommerce shortcodes
 *
 * PHP version 7
 *
 * @category WordPress_Theme
 * @package  ThemeCustomization
 *
 * @author  WProDevs <info@wprodevs.com>
 * @license https://www.gnu.org/licenses/gpl-3.0.html GPLv3 or later
 *
 * @link none
 */

namespace WooCommerce;

/**
 * This class is used to add custom WooCommerce shortcodes
 *
 * @category WordPress_Theme
 * @package  ThemeCustomization
 *
 * @author  WProDevs <info@wprodevs.com>
 * @license https://www.gnu.org/licenses/gpl-3.0.html GPLv3 or later
 *
 * @link none
 */
final class WooCommerceShortcodes
{
    /**
     * Stores a private copy of self instance used for singleton
     *
     * @var stdClass
     */
    private static $_instance;

    /**
     * Setup initial process after instantiation
     */
    public function __construct()
    {
        $this->_initialize();
    }

    /**
     * Add Shortcode actions
     *
     * @return none
     */
    private function _initialize()
    {
        add_shortcode('product_attributes', [$this, 'productAttributes']);
    }

    /**
     * Add custom shortcode which acts like [product_categories] shortcode
     *
     * @param array $atts Shortcode attributes
     *
     * @see  \WC_Shortcodes::product_categories()
     * @link https://github.com/woocommerce/woocommerce/blob/master/includes/class-wc-shortcodes.php#L149
     *
     * @return string
     */
    public function productAttributes($atts)
    {
        if (isset($atts['number'])) {
            $atts['limit'] = $atts['number'];
        }
        $atts = shortcode_atts(
            [
                'limit'      => '-1',
                'orderby'    => 'name',
                'order'      => 'ASC',
                'columns'    => '4',
                'hide_empty' => 1,
                'parent'     => 0,
                'ids'        => '',
                'attribute'  => '' // added custom attribute to the shortcode
            ],
            $atts,
            'product_attributes'
        );
        $ids        = array_filter(array_map('trim', explode(',', $atts['ids'])));
        $hide_empty = (true === $atts['hide_empty'] || 'true' === $atts['hide_empty'] || 1 === $atts['hide_empty'] || '1' === $atts['hide_empty']) ? 1 : 0;
        // Get terms and workaround WP bug with parents/pad counts.
        $args = [
            'orderby'    => $atts['orderby'],
            'order'      => $atts['order'],
            'hide_empty' => $hide_empty,
            'include'    => $ids,
            'pad_counts' => true,
            'child_of'   => $atts['parent'],
        ];
        $product_attributes = get_terms($atts['attribute'], $args); // replaced 'product_cat' with $atts['attribute']
        if ('' !== $atts['parent']) {
            $product_attributes = wp_list_filter(
                $product_attributes,
                [
                    'parent' => $atts['parent'],
                ]
            );
        }
        if ($hide_empty) {
            foreach ($product_attributes as $key => $category) {
                if (0 === $category->count) {
                    unset($product_attributes[ $key ]);
                }
            }
        }
        $atts['limit'] = '-1' === $atts['limit'] ? null : intval($atts['limit']);
        if ($atts['limit']) {
            $product_attributes = array_slice($product_attributes, 0, $atts['limit']);
        }
        $columns = absint($atts['columns']);
        wc_set_loop_prop('columns', $columns);
        wc_set_loop_prop('is_shortcode', true);
        ob_start();
        if ($product_attributes) {
            woocommerce_product_loop_start();
            foreach ($product_attributes as $category) {
                $GLOBALS['wpro_cat'] = $category;
                remove_action('woocommerce_before_subcategory_title', 'woocommerce_subcategory_thumbnail', 10);
                add_action('woocommerce_before_subcategory_title', [$this, 'echoAttributeImage'], 10);
                wc_get_template(
                    'content-product_cat.php',
                    [
                        'category' => $category,
                    ]
                );
                remove_action('woocommerce_before_subcategory_title', [$this, 'echoAttributeImage'], 10);
            }
            woocommerce_product_loop_end();
        }
        woocommerce_reset_loop();

        return '<div class="woocommerce columns-' . $columns . '">' . ob_get_clean() . '</div>';
    }

    /**
     * Echo product attribute image
     *
     * @return none
     */
    public function echoAttributeImage()
    {
        $category = $GLOBALS['wpro_cat'];
        unset($GLOBALS['wpro_cat']);
        $options   = get_option('taxonomy_image_plugin');
        $thumbnail = wc_placeholder_img_src();
        if (isset($options[$category->term_id])) {
            $attachment_id = $options[$category->term_id];
            $thumbnail     = wp_get_attachment_image_src($attachment_id, 'full')[0];
        }
        $small_thumbnail_size = apply_filters('subcategory_archive_thumbnail_size', 'woocommerce_thumbnail');
        $dimensions           = wc_get_image_size($small_thumbnail_size);
        echo '<img src="' . esc_url($thumbnail) . '" alt="' . esc_attr($category->name) . '" width="' . esc_attr($dimensions['width']) . '" height="' . esc_attr($dimensions['height']) . '" />';
    }

    /**
     * Singleton implementation to only allow 1 instance
     *
     * @return stdClass
     */
    public static function getInstance()
    {
        if (!isset(self::$_instance) || !self::$_instance instanceof self) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }
}
